/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 * @{
 */

/** @file
 */

#include <async.h>
#include <stdio.h>
#include <task.h>
#include <ipc/ipc.h>

#include "err.h"
#include "modules.h"

extern int	module_call( ipc_callid_t callid );
extern int	module_message( ipc_callid_t callid, ipcarg_t method, ipcarg_t arg1, ipcarg_t arg2, ipcarg_t arg3, ipcarg_t * result1, ipcarg_t * result2, ipcarg_t * result3 );
extern void	module_print_name( void );
extern int	module_start( void ( * client_connection )( ipc_callid_t iid, ipc_call_t * icall ));

void client_connection( ipc_callid_t iid, ipc_call_t * icall );
int main( int argc, char * argv[] );

/** Default thread for new connections.
 */
void client_connection( ipc_callid_t iid, ipc_call_t * icall ){
	ipc_callid_t callid;
	ipc_call_t call;
	ipcarg_t arg1, arg2, arg3;
	int res;

	/* Accept the connection */
	ipc_answer_0( iid, EOK );

	while( true ){
		callid = async_get_call( & call );
		arg1 = 0;
		arg2 = 0;
		arg3 = 0;
		res = module_call( callid );
		if( res == EOK ){
			res = module_message( callid, IPC_GET_METHOD( call ), IPC_GET_ARG1( call ), IPC_GET_ARG2( call ), IPC_GET_ARG3( call ), & arg1, & arg2, & arg3 );
		}
		ipc_answer_2( callid, res, arg1, arg2 );
	}
}

/**	Starts the module.
 *	Parameters are ignored.
 */
int main( int argc, char * argv[] ){
	ERROR_DECLARE;

	printf("\nTask %d - ", task_get_id());
	module_print_name();
	if( ERROR_OCCURED( module_start( client_connection ))){
		printf( " - ERROR %i", ERROR_CODE );
		return ERROR_CODE;
	}
	return EOK;
}

/** @}
 */
